"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.getParts = exports.isOptions = void 0;
const aws_sdk_1 = require("aws-sdk");
const protopath_1 = require("protopath");
const apiVersion = '2014-11-06';
function isOptions(o) {
    return (typeof o === 'object' &&
        (!o.region || typeof o.region === 'string') &&
        typeof o.access === 'string' &&
        typeof o.secret === 'string');
}
exports.isOptions = isOptions;
function getS3(options) {
    if (!options)
        return new aws_sdk_1.S3({ apiVersion });
    const { region, access: accessKeyId, secret: secretAccessKey } = options;
    return new aws_sdk_1.S3({ apiVersion, region, accessKeyId, secretAccessKey });
}
function getParts(path) {
    const parts = protopath_1.popSlash(path).split('/');
    const bucket = parts[0];
    if (!bucket)
        throw new Error('No S3 bucket supplied');
    const object = parts.slice(1).join('/');
    return [bucket, object.length ? object : undefined];
}
exports.getParts = getParts;
function create(options) {
    const s3 = getS3(options);
    async function read(path) {
        const [Bucket, Key] = getParts(path);
        if (!Key)
            throw new Error('Invalid S3 read - no object specified');
        const { Body: buffer } = await s3.getObject({ Bucket, Key }).promise();
        return buffer;
    }
    async function write(path, data) {
        const [Bucket, Key] = getParts(path);
        if (!Key)
            throw new Error('Invalid S3 write - no object specified');
        await s3.putObject({ Bucket, Key, Body: data }).promise();
    }
    async function createReadStream(path) {
        const [Bucket, Key] = getParts(path);
        if (!Key)
            throw new Error('Invalid S3 read - no object specified');
        const request = s3.getObject({ Bucket, Key });
        return request.createReadStream().on('error', () => request.abort());
    }
    async function writeStream(path, data) {
        const [Bucket, Key] = getParts(path);
        if (!Key)
            throw new Error('Invalid S3 write - no object specified');
        await s3.upload({ Bucket, Key, Body: data }).promise();
    }
    async function list(path) {
        const [Bucket, Key] = getParts(path);
        const Prefix = Key ? `${Key}/` : '';
        let ContinuationToken;
        let list = [];
        do {
            const params = { Bucket, Prefix, Delimiter: '/', ContinuationToken };
            const response = await s3.listObjectsV2(params).promise();
            if (!response.CommonPrefixes || !response.Contents) {
                throw new Error('Unexpected S3 list response');
            }
            list = [
                ...list,
                ...response.CommonPrefixes.map((v) => ({
                    type: 'directory',
                    path: protopath_1.popSlash(v.Prefix.slice(Prefix.length)),
                })),
                ...response.Contents.map((v) => ({
                    type: 'file',
                    path: v.Key.slice(Prefix.length),
                    size: v.Size,
                })),
            ];
            ContinuationToken = response.NextContinuationToken;
        } while (ContinuationToken);
        return list.sort((a, b) => a.path.localeCompare(b.path));
    }
    return {
        read,
        write,
        createReadStream,
        writeStream,
        list,
    };
}
exports.create = create;
